import http from 'http';
import { nanoid } from 'nanoid';
import { Pull, Push } from 'zeromq';

const host = 'localhost';
const port = 3000;

const sender = new Push();
await sender.bind(`tcp://127.0.0.1:5557`);
const receiver = new Pull();
await receiver.bind(`tcp://127.0.0.1:5558`);

const generateId = () => nanoid(32);

const clients: Record<string, http.ServerResponse> = {};

const server = http.createServer((req, res) => {
  switch (req.url) {
    case '/run':
      const jobId = generateId();
      res.writeHead(200, {
        'Content-Type': 'text/event-stream',
        Connection: 'keep-alive',
        'Cache-Control': 'no-cache',
      });
      sender.send(`${jobId}echo a`).then(() => {
        res.write('event: connected\n');
        res.write(`data: ${jobId}\n`);
        res.write('id: 0\n');
      });
      req.on('close', () => {
        res.end('OK');
        delete clients[jobId];
      });
      clients[jobId] = res;
      break;
    default:
      res.writeHead(404);
      res.end('404!');
  }
});
server.listen(port, () => {
  console.log(`Server is running on http://${host}:${port}`);
});

for await (const [buff] of receiver) {
  const jobId = buff.slice(0, 32).toString();
  console.log(`Received ${jobId}`);
  const res = clients[jobId];
  if (!res) {
    continue;
  }
  res.write('event: message\n');
  res.write(`data: ${encodeURIComponent(buff.slice(32).toString())}\n`);
  res.write('id: 0\n');
}
